@@ -10,13 +10,12 @@ from __future__ import annotations  # Make typing work for the enclosing class
 
 __author__ = "Jonas Van Der Donckt, Emiel Deprost, Jeroen Van Der Donckt"
 
-import logging
-import warnings
-from pathlib import Path
-from typing import Dict, List, Optional, Tuple, Union
-
 import dill
 import pandas as pd
+import numpy as np
+
+from pathlib import Path
+from typing import Dict, List, Optional, Tuple, Union
 from pathos.multiprocessing import ProcessPool
 from tqdm.auto import tqdm
 
@@ -25,7 +24,8 @@ from .logger import logger
 from .strided_rolling import StridedRolling
 from ..features.function_wrapper import NumpyFuncWrapper
 from ..utils.data import to_list, to_series_list, flatten
-from ..utils.timedelta import timedelta_to_str
+from ..utils.time import timedelta_to_str
+from ..utils.logging import delete_logging_handlers, add_logging_handler
 
 
 class FeatureCollection:
@@ -42,8 +42,9 @@ class FeatureCollection:
         self,
         feature_descriptors: Optional[
             Union[
-                FeatureDescriptor, MultipleFeatureDescriptors,
-                List[Union[FeatureDescriptor, MultipleFeatureDescriptors]]
+                FeatureDescriptor,
+                MultipleFeatureDescriptors,
+                List[Union[FeatureDescriptor, MultipleFeatureDescriptors]],
             ]
         ] = None,
     ):
@@ -71,16 +72,13 @@ class FeatureCollection:
 
         """
         return list(
-            set(
-                flatten(
-                    [fr_key[0] for fr_key in self._feature_desc_dict.keys()]
-                )
-            )
+            set(flatten([fr_key[0] for fr_key in self._feature_desc_dict.keys()]))
         )
 
     @staticmethod
-    def _get_collection_key(feature: FeatureDescriptor)\
-            -> Tuple[tuple, pd.Timedelta, pd.Timedelta]:
+    def _get_collection_key(
+        feature: FeatureDescriptor,
+    ) -> Tuple[tuple, pd.Timedelta, pd.Timedelta]:
         # Note: `window` & `stride` properties can either be a pd.Timedelta or an int
         return feature.series_name, feature.window, feature.stride
 
@@ -102,9 +100,12 @@ class FeatureCollection:
     def add(
         self,
         features: Union[
-            FeatureDescriptor, MultipleFeatureDescriptors, FeatureCollection, List[
+            FeatureDescriptor, 
+            MultipleFeatureDescriptors,
+            FeatureCollection,
+            List[
                 Union[FeatureDescriptor, MultipleFeatureDescriptors, FeatureCollection]
-            ]
+            ],
         ],
     ):
         """Add feature(s) to the FeatureCollection.
@@ -137,37 +138,48 @@ class FeatureCollection:
 
     @staticmethod
     def _executor(idx: int):
-        stroll, function = stroll_feat_list[idx]
+        # global get_stroll_func
+        stroll, function = get_stroll_func(idx)
         return stroll.apply_func(function)
 
-    def _construct_stroll_feat_list(
+    def _stroll_feat_generator(
         self, series_dict: Dict[str, pd.Series], window_idx: str, approve_sparsity: bool
     ) -> List[Tuple[StridedRolling, NumpyFuncWrapper]]:
         # --- Future work ---
         # We could also make the StridedRolling creation multithreaded
         # Very low priority because the STROLL __init__ is rather efficient!
-        stroll_feat_list: List[Tuple[StridedRolling, NumpyFuncWrapper]] = []
-        for key, win, stride in self._feature_desc_dict.keys():
-            try:
-                stroll = StridedRolling(
-                    data=[series_dict[k] for k in key],
-                    window=win,
-                    stride=stride,
-                    window_idx=window_idx,
-                    approve_sparsity=approve_sparsity,
-                )
-            except KeyError:
-                raise KeyError(f"Key {key} not found in series dict.")
+        keys_wins_strides = list(self._feature_desc_dict.keys())
+        lengths = np.cumsum(
+            [len(self._feature_desc_dict[k]) for k in keys_wins_strides]
+        )
 
-            for feature in self._feature_desc_dict[(key, win, stride)]:
-                stroll_feat_list.append((stroll, feature.function))
-        return stroll_feat_list
+        def get_stroll_function(idx):
+            key_idx = np.searchsorted(lengths, idx, "right")  # right bc idx starts at 0
+            key, win, stride = keys_wins_strides[key_idx]
+            stroll = StridedRolling(
+                data=[series_dict[k] for k in key],
+                window=win,
+                stride=stride,
+                window_idx=window_idx,
+                approve_sparsity=approve_sparsity,
+            )
+            feature = self._feature_desc_dict[keys_wins_strides[key_idx]][
+                idx - lengths[key_idx]
+            ]
+            return stroll, feature.function
+
+        return get_stroll_function
+
+    def _get_stroll_feat_length(self) -> int:
+        return sum(
+            len(self._feature_desc_dict[k]) for k in self._feature_desc_dict.keys()
+        )
 
     def calculate(
         self,
         data: Union[pd.Series, pd.DataFrame, List[Union[pd.Series, pd.DataFrame]]],
         return_df: Optional[bool] = False,
-        window_idx: Optional[str] = 'end',
+        window_idx: Optional[str] = "end",
         approve_sparsity: Optional[bool] = False,
         show_progress: Optional[bool] = False,
         logging_file_path: Optional[Union[str, Path]] = None,
@@ -192,7 +204,7 @@ class FeatureCollection:
             **Remark**: each Series / DataFrame must have a `pd.DatetimeIndex`.
             **Remark**: we assume that each name / column is unique.
         return_df : bool, optional
-            Whether the output needs to be a dataframe list or a DataFrame, by default 
+            Whether the output needs to be a dataframe list or a DataFrame, by default
             False.
             If `True` the output dataframes will be merged to a DataFrame with an outer
             merge.
@@ -202,7 +214,7 @@ class FeatureCollection:
             by default 'end'. All features in this collection will use the same
             window_idx.
         approve_sparsity: bool, optional
-            Bool indicating whether the user acknowledges that there may be sparsity 
+            Bool indicating whether the user acknowledges that there may be sparsity
             (i.e., irregularly sampled data), by default False.
             If False and sparsity is observed, a warning is raised.
         show_progress: bool, optional
@@ -239,31 +251,10 @@ class FeatureCollection:
 
         """
         # Delete other logging handlers
-        if len(logger.handlers) > 1:
-            logger.handlers = [
-                h for h in logger.handlers if type(h) == logging.StreamHandler
-            ]
-        assert len(logger.handlers) == 1, "Multiple logging StreamHandlers present!!"
-
+        delete_logging_handlers(logger)
+        # Add logging handler (if path provided)
         if logging_file_path:
-            if not isinstance(logging_file_path, Path):
-                logging_file_path = Path(logging_file_path)
-            if logging_file_path.exists():
-                warnings.warn(
-                    f"Logging file ({logging_file_path}) already exists. "
-                    f"This file will be overwritten!"
-                )
-                # Clear the file
-                #  -> because same FileHandler is used when calling this method twice
-                open(logging_file_path, "w").close()
-            f_handler = logging.FileHandler(logging_file_path, mode="w")
-            f_handler.setFormatter(
-                logging.Formatter(
-                    "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
-                )
-            )
-            f_handler.setLevel(logging.INFO)
-            logger.addHandler(f_handler)
+            add_logging_handler(logger, logging_file_path)
 
         # Convert the data to a series_dict
         series_dict: Dict[str, pd.Series] = {}
@@ -276,34 +267,28 @@ class FeatureCollection:
                 series_dict[str(s.name)] = s
 
         calculated_feature_list: List[pd.DataFrame] = []
+        
         # Note: this variable has a global scope so this is shared in multiprocessing
-        global stroll_feat_list
-        stroll_feat_list = self._construct_stroll_feat_list(
+        global get_stroll_func
+        get_stroll_func = self._stroll_feat_generator(
             series_dict, window_idx, approve_sparsity
         )
 
         if n_jobs in [0, 1]:
-            # print('Executing feature extraction sequentially')
+            idxs = range(self._get_stroll_feat_length())
             if show_progress:
-                stroll_feat_list = tqdm(stroll_feat_list)
-            for stroll, func in stroll_feat_list:
-                calculated_feature_list.append(stroll.apply_func(func))
+                idxs = tqdm(idxs)
+            calculated_feature_list = [self._executor(idx) for idx in idxs]
         else:
-            # ---- Future work -----
-            # Try locking inside the executer when calling next() on a global generator
-            # Create global (precomputed) stroll-feature list 
-            # global stroll_feat_list
-            # stroll_feat_list = [stroll_feat for stroll_feat in stroll_feat_generator]
             # https://pathos.readthedocs.io/en/latest/pathos.html#usage
             with ProcessPool(nodes=n_jobs, source=True) as pool:
                 results = pool.uimap(
                     self._executor,
-                    range(len(stroll_feat_list)),
+                    range(self._get_stroll_feat_length()),
                 )
                 if show_progress:
-                    results = tqdm(results, total=len(stroll_feat_list))
-                for f in results:
-                    calculated_feature_list.append(f)
+                    results = tqdm(results, total=self._get_stroll_feat_length())
+                calculated_feature_list = [f for f in results]
                 # Close & join - see: https://github.com/uqfoundation/pathos/issues/131
                 pool.close()
                 pool.join()
@@ -311,16 +296,7 @@ class FeatureCollection:
                 pool.clear()
 
         if return_df:
-            df_merged = pd.DataFrame()
-            for calculated_feature in calculated_feature_list:
-                df_merged = pd.merge(
-                    left=df_merged,
-                    right=calculated_feature,
-                    how="outer",
-                    left_index=True,
-                    right_index=True,
-                )
-            return df_merged
+            return pd.concat(calculated_feature_list, axis=1, join="outer", copy=False)
         else:
             return calculated_feature_list
 
