@@ -14,16 +14,16 @@ __author__ = "Jonas Van Der Donckt, Jeroen Van Der Donckt, Emiel Deprost"
 
 import time
 import warnings
-import numpy as np
 import pandas as pd
+import numpy as np
 
-from typing import Callable, Union, List, Tuple, Optional
 from collections import namedtuple
+from typing import Callable, Union, List, Tuple, Optional
 
 from .function_wrapper import NumpyFuncWrapper
 from .logger import logger
 from ..utils.data import to_series_list
-from ..utils.timedelta import timedelta_to_str
+from ..utils.time import timedelta_to_str
 
 
 class StridedRolling:
@@ -94,31 +94,35 @@ class StridedRolling:
 
         # And slice **all** the series to these tightest bounds
         assert (t_end - t_start) > window
-        series_list = [s[t_start:t_end] for s in series_list]
+        if len(series_list) > 1:
+            series_list = [s[t_start:t_end] for s in series_list]
 
         # 2. Create the time_index which will be used for DataFrame reconstruction
-        # use closed = left to exclude 'end' if it falls on the boundary
-        # note: the index automatically takes the timezone of `t_start` & `t_end`
-        # note: the index-name of the first passed series will be used
-        self.index = pd.date_range(
-            t_start, t_end - window, freq=stride, name=series_list[0].index.name
-        )
-
-        # --- and adjust the time_index
+        # 2.1 - and adjust the time_index
         # note: this code can also be placed in the `apply_func` method (if we want to
         #  make the bound window-idx setting feature specific).
         if window_idx == "end":
-            self.index += window
+            window_idx_offset = window
         elif window_idx == "middle":
-            self.index += window / 2
+            window_idx_offset = window / 2
         elif window_idx == "begin":
-            pass
+            window_idx_offset = pd.Timedelta(seconds=0)
         else:
             raise ValueError(
                 f"window index {window_idx} must be either of: "
                 "['end', 'middle', 'begin']"
             )
 
+        # use closed = left to exclude 'end' if it falls on the boundary
+        # note: the index automatically takes the timezone of `t_start` & `t_end`
+        # note: the index-name of the first passed series will be used
+        self.index = pd.date_range(
+            start=t_start + window_idx_offset,
+            end=t_end - window + window_idx_offset,
+            freq=stride,
+            name=series_list[0].index.name
+        )
+
         # ---------- Efficient numpy code -------
         # 1. Convert everything to int64
         np_start = t_start.to_datetime64()
@@ -164,7 +168,7 @@ class StridedRolling:
                     last_container.end_indexes - last_container.start_indexes, q=qs
                 )
                 q_str = ", ".join([f"q={q}: {v}" for q, v in zip(qs, series_idx_stats)])
-                if series_idx_stats[0] != series_idx_stats[1]:  # min != max
+                if not all(series_idx_stats == series_idx_stats[-1]):  # min != max
                     warnings.warn(
                         f"There are gaps in the time-series {series.name}; "
                         + f"\n \t Quantiles of nb values in window: {q_str}",
@@ -257,17 +261,15 @@ class StridedRolling:
 
         t_start = time.time()
 
-        def get_slices(idx):
-            # get the slice of each series for the given index
-            return [
-                sc.values[sc.start_indexes[idx]: sc.end_indexes[idx]]
-                for sc in self.series_containers
-            ]
-
         # --- Future work ---
         # would be nice if we could optimize this double for loop with something
         # more vectorized
-        out = np.array([np_func(*get_slices(idx)) for idx in range(len(self.index))])
+        out = np.array(
+            [np_func(
+                *[sc.values[sc.start_indexes[idx]: sc.end_indexes[idx]]
+                  for sc in self.series_containers]
+            ) for idx in range(len(self.index))]
+        )
 
         # Aggregate function output in a dictionary
         feat_out = {}
